package org.processmining.acceptingpetrinetminer.algorithms;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetArrayFactory;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinetclassicalreductor.algorithms.ReduceUsingMurataRulesAlgorithm;
import org.processmining.acceptingpetrinetclassicalreductor.parameters.ReduceUsingMurataRulesParameters;
import org.processmining.acceptingpetrinetminer.miners.DecomposedMiner;
import org.processmining.acceptingpetrinetminer.miners.DecomposedMinerManager;
import org.processmining.acceptingpetrinetminer.miners.DecomposedMinerParameters;
import org.processmining.acceptingpetrinetminer.parameters.DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.log.models.EventLogArray;

public class DiscoverAcceptingPetriNetArrayFromEventLogArrayAlgorithm {

	public AcceptingPetriNetArray apply(PluginContext context, EventLogArray logs,
			DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters parameters) {
		AcceptingPetriNetArray nets = AcceptingPetriNetArrayFactory.createAcceptingPetriNetArray();
		nets.init();
		for (int i = 0; i < logs.getSize(); i++) {
//			context.log("Discovering net " + (i + 1) + "/" + logs.getSize());
			AcceptingPetriNet acceptingNet = AcceptingPetriNetFactory.createAcceptingPetriNet();
			DecomposedMinerParameters decomposedParameters = new DecomposedMinerParameters(logs.getLog(i));
			decomposedParameters.setClassifier(parameters.getClassifier());
			decomposedParameters.setNrOfClusters(logs.getSize());
			DecomposedMiner miner = DecomposedMinerManager.getInstance().getMiner(parameters.getMiner());
			acceptingNet = miner.mine(context, logs.getLog(i), decomposedParameters);
			
			ReduceUsingMurataRulesAlgorithm reductor = new ReduceUsingMurataRulesAlgorithm();
			ReduceUsingMurataRulesParameters reductorParameters = new ReduceUsingMurataRulesParameters();
			AcceptingPetriNet reducedNet = reductor.apply(null, acceptingNet, reductorParameters);
			if (reducedNet != null) {
				acceptingNet = reducedNet;
			}
			
			nets.addNet(acceptingNet);
		}
		return nets;
	}
}
