package org.processmining.acceptingpetrinetminer.miners.impl;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.acceptingpetrinetminer.miners.DecomposedImplicitMiner;
import org.processmining.acceptingpetrinetminer.miners.DecomposedMinerParameters;
import org.processmining.causalactivitygraph.models.CausalActivityGraph;
import org.processmining.causalactivitygraphcreator.parameters.ConvertCausalActivityMatrixToCausalActivityGraphParameters;
import org.processmining.causalactivitygraphcreator.plugins.ConvertCausalActivityMatrixToCausalActivityGraphPlugin;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrixminer.miners.MatrixMiner;
import org.processmining.causalactivitymatrixminer.miners.MatrixMinerParameters;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.packages.UnknownPackageException;
import org.processmining.framework.packages.UnknownPackageTypeException;
import org.processmining.framework.packages.impl.CancelledException;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.hybridilpminer.algorithms.miners.LPMiner;
import org.processmining.hybridilpminer.algorithms.miners.LPMinerFactory;
import org.processmining.hybridilpminer.parameters.DiscoveryStrategy;
import org.processmining.hybridilpminer.parameters.DiscoveryStrategyType;
import org.processmining.hybridilpminer.parameters.LPConstraintType;
import org.processmining.hybridilpminer.parameters.LPFilter;
import org.processmining.hybridilpminer.parameters.LPFilterType;
import org.processmining.hybridilpminer.parameters.LPObjectiveType;
import org.processmining.hybridilpminer.parameters.LPVariableType;
import org.processmining.hybridilpminer.parameters.NetClass;
import org.processmining.hybridilpminer.parameters.XLogHybridILPMinerParametersImpl;
import org.processmining.lpengines.interfaces.LPEngine;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

public abstract class DecomposedAbstractHybridILPMiner implements DecomposedImplicitMiner {

	public Petrinet mineImplicit(PluginContext context, XLog eventLog, DecomposedMinerParameters parameters)
			throws CancelledException, UnknownPackageTypeException, UnknownPackageException, ConnectionCannotBeObtained {
		MatrixMiner miner = getMiner();
		MatrixMinerParameters minerParameters = new MatrixMinerParameters(eventLog);
		minerParameters.setClassifier(parameters.getClassifier());
		CausalActivityMatrix matrix = miner.mineMatrix(context, eventLog, minerParameters);
		ConvertCausalActivityMatrixToCausalActivityGraphPlugin creator = new ConvertCausalActivityMatrixToCausalActivityGraphPlugin();
		ConvertCausalActivityMatrixToCausalActivityGraphParameters creatorParameters = new ConvertCausalActivityMatrixToCausalActivityGraphParameters();
		creatorParameters.setZeroValue(miner.getZeroValue());
		creatorParameters.setConcurrencyRatio(miner.getConcurrencyRatio());
		creatorParameters.setIncludeThreshold(miner.getIncludeThreshold());
		CausalActivityGraph graph = creator.run(context, matrix, creatorParameters);
		DiscoveryStrategy strategy = new DiscoveryStrategy(getMiningStrategy());
		LPFilter filter = new LPFilter(getFilter(), getFilter().getDefaultThreshold());
		XLogHybridILPMinerParametersImpl configuration = new XLogHybridILPMinerParametersImpl
				(context, getEngineType(), strategy, NetClass.PT_NET, getConstraints(),
						getObjectiveType(), getVariableType(), filter, true, eventLog, parameters.getClassifier());
		configuration.getDiscoveryStrategy().setCausalActivityGraph(graph);
		LPMiner lpMiner = LPMinerFactory.createLPMiner(configuration, null);
		lpMiner.run();
		return lpMiner.synthesizeNet().getFirst();
	}

	public String getName() {
		return "Hybrid ILP MIner";
	}

	public abstract MatrixMiner getMiner();
	
	public LPEngine.EngineType getEngineType() {
		return LPEngine.EngineType.LPSOLVE;
	}

	public DiscoveryStrategyType getMiningStrategy() {
		return DiscoveryStrategyType.CAUSAL;
	}

	public Set<LPConstraintType> getConstraints() {
		Set<LPConstraintType> constraints = new HashSet<>();
		constraints.addAll(NetClass.PT_NET.getRequiredConstraints());
		constraints.add(LPConstraintType.EMPTY_AFTER_COMPLETION);
		return constraints;
	}

	public LPObjectiveType getObjectiveType() {
		return LPObjectiveType.WEIGHTED_ABSOLUTE_PARIKH;
	}

	public LPVariableType getVariableType() {
		return LPVariableType.DUAL;
	}

	public LPFilterType getFilter() {
		return LPFilterType.NONE;
	}

	public boolean requiresStartEndEvents() {
		return false;
	}
}
