package org.processmining.acceptingpetrinetminer.miners.impl;

import java.util.HashSet;
import java.util.Set;

import org.deckfour.xes.model.XLog;
import org.processmining.acceptingpetrinetminer.miners.DecomposedImplicitMiner;
import org.processmining.acceptingpetrinetminer.miners.DecomposedMinerParameters;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.packages.PackageManager;
import org.processmining.framework.packages.UnknownPackageException;
import org.processmining.framework.packages.UnknownPackageTypeException;
import org.processmining.framework.packages.impl.CancelledException;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.etm.fitness.TreeFitnessInfo;
import org.processmining.plugins.etm.fitness.TreeFitnessInfo.Dimension;
import org.processmining.plugins.etm.fitness.metrics.FitnessReplay;
import org.processmining.plugins.etm.parameters.ETMParam;
import org.processmining.plugins.etm.parameters.ETMParamFactory;
import org.processmining.plugins.etm.termination.TerminateAtQDimValue;
import org.processmining.processtree.ProcessTree;
import org.uncommonseditedbyjoosbuijs.watchmaker.framework.TerminationCondition;

public class DecomposedETMiner implements DecomposedImplicitMiner {

	private static final String NAME = "Evolutionary Tree Miner";

	public Petrinet mineImplicit(PluginContext context, XLog eventLog, DecomposedMinerParameters parameters)
			throws CancelledException, UnknownPackageTypeException, UnknownPackageException, ConnectionCannotBeObtained {
		
		/*
		 * Info for evaluator that checks for duplicate activities.
		 */
		@SuppressWarnings("unchecked")
		final TreeFitnessInfo dupInfo = new TreeFitnessInfo(FitnessReplay.class, "DP", "Duplicates",
				"Duplicate evaluator", Dimension.OTHER, true);
		
		PackageManager.getInstance().findOrInstallPackages("EvolutionaryTreeMiner");
		
		/*
		 * Create initial parameters.
		 */
		final ETMParam param = ETMParamFactory.buildStandardParam(eventLog); 

		/*
		 * Have the ETM stop when fitness replay and no-duplicates 
		 * both reach 1.0.
		 */
		Set<Pair<TreeFitnessInfo, Double>> stopAt = new HashSet<Pair<TreeFitnessInfo, Double>>(2);
		stopAt.add(new Pair<TreeFitnessInfo, Double>(dupInfo, 1.0));
		stopAt.add(new Pair<TreeFitnessInfo, Double>(FitnessReplay.info, 1.0));
		TerminationCondition terminationCondition = new TerminateAtQDimValue(param.getCentralRegistry(), stopAt);
		param.addTerminationCondition(terminationCondition);

		/*
		 * Start the ETM miner.
		 */
		ProcessTree pt = context.tryToFindOrConstructFirstNamedObject(ProcessTree.class,
				"Mine a Process Tree with ETMd using parameters and classifier", null, null, eventLog, parameters.getClassifier(), param);
			
		/*
		 * Convert extended tree into a Petri net.
		 */
		Petrinet pn = context.tryToFindOrConstructFirstNamedObject(Petrinet.class,
				"Convert Process Tree to Petri Net", null, null, pt);
		
		return pn;
	}

	public String getName() {
		return NAME;
	}

	public boolean requiresStartEndEvents() {
		return false;
	}
}
