package org.processmining.acceptingpetrinetminer.miners.impl;

import org.deckfour.xes.model.XLog;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinet.parameters.PackPetrinetParameters;
import org.processmining.acceptingpetrinet.parameters.UndoubleAcceptingPetriNetParameters;
import org.processmining.acceptingpetrinet.plugins.PackPetrinetPlugin;
import org.processmining.acceptingpetrinet.plugins.UndoubleAcceptingPetriNetPlugin;
import org.processmining.acceptingpetrinetminer.miners.DecomposedImplicitMiner;
import org.processmining.acceptingpetrinetminer.miners.DecomposedMiner;
import org.processmining.acceptingpetrinetminer.miners.DecomposedMinerParameters;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.packages.UnknownPackageException;
import org.processmining.framework.packages.UnknownPackageTypeException;
import org.processmining.framework.packages.impl.CancelledException;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

/*
 * Creates a DecomposedMiner from a DecomposedImplicitMiner.
 */
public class DecomposedWrappedMiner implements DecomposedMiner {

	/*
	 * The implicit miner.
	 */
	private DecomposedImplicitMiner miner;
	
	public DecomposedWrappedMiner(DecomposedImplicitMiner miner) {
		this.miner = miner;
	}
	
	/*
	 * Mine the log using the implicit miner, create initial and final markings, and undouble the net.
	 * (non-Javadoc)
	 * @see org.processmining.acceptingpetrinetminer.miners.DecomposedMiner#mine(org.processmining.framework.plugin.PluginContext, org.deckfour.xes.model.XLog, org.processmining.acceptingpetrinetminer.miners.DecomposedMinerParameters)
	 */
	public AcceptingPetriNet mine(PluginContext context, XLog eventLog, DecomposedMinerParameters parameters) {
		try {
			/*
			 * Run the miner.
			 */
			Petrinet pn = miner.mineImplicit(context, eventLog, parameters);
			/*
			 * Convert the Petri net to an Accepting Petri net.
			 */
			PackPetrinetParameters packParams = new PackPetrinetParameters();
			packParams.setTryConnections(false);
			AcceptingPetriNet apn = (new PackPetrinetPlugin()).runDefault(context, pn);
			UndoubleAcceptingPetriNetParameters undoubleParams = new UndoubleAcceptingPetriNetParameters();
			undoubleParams.setTryConnections(false);
			return (new UndoubleAcceptingPetriNetPlugin()).run(context, apn, undoubleParams);
		} catch (CancelledException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnknownPackageTypeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnknownPackageException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ConnectionCannotBeObtained e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return AcceptingPetriNetFactory.createAcceptingPetriNet();
	}

	public String getName() {
		return miner.getName();
	}

	public boolean requiresStartEndEvents() {
		return miner.requiresStartEndEvents();
	}
}
