package org.processmining.acceptingpetrinetminer.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNetArray;
import org.processmining.acceptingpetrinetminer.algorithms.DiscoverAcceptingPetriNetArrayFromEventLogArrayAlgorithm;
import org.processmining.acceptingpetrinetminer.connections.DiscoverAcceptingPetriNetArrayFromEventLogArrayConnection;
import org.processmining.acceptingpetrinetminer.dialogs.DiscoverAcceptingPetriNetArrayFromEventLogArrayDialog;
import org.processmining.acceptingpetrinetminer.help.DiscoverAcceptingPetriNetArrayFromEventLogArrayHelp;
import org.processmining.acceptingpetrinetminer.parameters.DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.log.models.EventLogArray;

@Plugin(name = "Discover Accepting Petri Nets", categories = { PluginCategory.Discovery }, parameterLabels = { "Event Log Array", "Parameters" }, returnLabels = { "Accepting Petri Net Array" }, returnTypes = { AcceptingPetriNetArray.class }, help = DiscoverAcceptingPetriNetArrayFromEventLogArrayHelp.TEXT)
public class DiscoverAcceptingPetriNetArrayFromEventLogArrayPlugin extends DiscoverAcceptingPetriNetArrayFromEventLogArrayAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Discover Accepting Petri Nets, UI", requiredParameterLabels = { 0 })
	public AcceptingPetriNetArray runUI(UIPluginContext context, EventLogArray logs) {
		DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters parameters = new DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters(
				logs);
		DiscoverAcceptingPetriNetArrayFromEventLogArrayDialog dialog = new DiscoverAcceptingPetriNetArrayFromEventLogArrayDialog(
				logs, parameters);
		InteractionResult result = context.showWizard("Configure discovery (classifier, miner)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnection(context, logs, parameters);
	}

	@Deprecated
	public AcceptingPetriNetArray discoverUI(UIPluginContext context, EventLogArray logs) {
		return runUI(context, logs);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Discover Accepting Petri Nets, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNetArray runDefault(PluginContext context, EventLogArray logs) {
		DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters parameters = new DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters(
				logs);
		return runConnection(context, logs, parameters);
	}

	@Deprecated
	public AcceptingPetriNetArray discoverDefault(PluginContext context, EventLogArray logs) {
		return runDefault(context, logs);
	}

	@PluginVariant(variantLabel = "Discover Accepting Petri Nets, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNetArray run(PluginContext context, EventLogArray logs,
			DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters parameters) {
		return runConnection(context, logs, parameters);
	}

	@Deprecated
	public AcceptingPetriNetArray discoverParameters(PluginContext context, EventLogArray logs,
			DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters parameters) {
		return run(context, logs, parameters);
	}

	private AcceptingPetriNetArray runConnection(PluginContext context, EventLogArray logs,
			DiscoverAcceptingPetriNetArrayFromEventLogArrayParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<DiscoverAcceptingPetriNetArrayFromEventLogArrayConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						DiscoverAcceptingPetriNetArrayFromEventLogArrayConnection.class, context, logs);
				for (DiscoverAcceptingPetriNetArrayFromEventLogArrayConnection connection : connections) {
					if (connection.getObjectWithRole(DiscoverAcceptingPetriNetArrayFromEventLogArrayConnection.LOGS)
							.equals(logs) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(DiscoverAcceptingPetriNetArrayFromEventLogArrayConnection.NETS);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNetArray nets = apply(context, logs, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new DiscoverAcceptingPetriNetArrayFromEventLogArrayConnection(logs, nets, parameters));
		}
		return nets;
	}


}
